/*
 * Copyright © 2020, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.administer;

import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.JPanel;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.MultilingualButton;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.window.manager.Control;



public class RegistryButton extends JPanel implements Babelfish {

  private static final long serialVersionUID = 4818417888360570711L;
  MultilingualButton buttonRegistrieren = new MultilingualButton();
  MultilingualButton buttonWeiter = new MultilingualButton();
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  private FlowLayout flow = new FlowLayout();
  ActionListener registrierenListener;
  private ActionListener schliessenListener;

  public RegistryButton() {
    setLayout(flow);
    add(buttonRegistrieren);
    registrierenListener = new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        changes.firePropertyChange(RegistryButton.class.getName(), Control.NULL, Control.REGISTRIEREN);
      }
    };
    schliessenListener = new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        changes.firePropertyChange(RegistryButton.class.getName(), Control.NULL, Control.WEITER);
      }
    };
    buttonRegistrieren.addActionListener(registrierenListener);
    add(buttonWeiter);
    buttonWeiter.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    buttonWeiter.addActionListener(schliessenListener);
  }



  public synchronized void addKnopfListener(PropertyChangeListener paramPropertyChangeListener) {
    changes.addPropertyChangeListener(paramPropertyChangeListener);
  }



  public void hitSign() {
    changes.firePropertyChange(RegistryButton.class.getName(), Control.NULL, Control.REGISTRIEREN);
  }



  public synchronized void removeAllKnopfListener() {
    for (PropertyChangeListener tmp : changes.getPropertyChangeListeners()) {
      removeKnopfListener(tmp);
    }
  }



  public synchronized void removeAllListener() {
    removeAllKnopfListener();
    buttonWeiter.removeActionListener(schliessenListener);
    buttonRegistrieren.removeActionListener(registrierenListener);
  }



  public synchronized void removeKnopfListener(PropertyChangeListener paramPropertyChangeListener) {
    changes.removePropertyChangeListener(paramPropertyChangeListener);
  }



  @Override
  public void setLanguage(ISO639 code) {
    buttonRegistrieren.setLanguage(KEY.BUTTON_RIGISTRATE, code);
    buttonWeiter.setLanguage(KEY.BUTTON_WEITER, code);
  }
}
