/*
 * Copyright © 2021, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.iptv;

import javax.swing.JButton;



/**
 * Der Programmauswahlknopf für einen Sender. In der Grundeinstellung ist dem
 * Knopf noch kein Sendercode und keine URL zugewiesen.
 * 
 * 
 * @author llange
 *
 */
public class IptvButton extends JButton {

  private static final long serialVersionUID = -1269908316447589195L;

  private Integer code;
  private String url;

  /**
   * Der Kopfname in der Senderauswahl.
   * 
   * 
   * @param sendername
   *                   ein Fernsehprogrammname
   */
  public IptvButton(String sendername) {
    super(sendername);
  }




  /**
   * Der Sendercode ist eine Zahl wie auf einer Fernbedienung. Der Rückgabewert
   * kann {@code null} sein, wenn der Knopf keinem Sender zugeordnet werden kann.
   * 
   * @return der Sendercode
   */
  public Integer getCode() {
    return code;
  }



  /**
   * Über diese Url wird das Programm empfangen. Der Rückgabewert kann
   * {@code null} sein, wenn der Knopf keinem Sender zugeordnet werden kann.
   * 
   * @return die Sender URL
   */
  public String getUrl() {
    return url;
  }



  public void setUrl(String value) {
    url = value;
  }


}
