/*
 * Copyright © 2020, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.phone;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.nexuswob.gui.TableModel3;
import org.nexuswob.util.Cell;
import org.nexuswob.util.Util;
import net.javacomm.client.resource.Resource;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.MultilingualLabel;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.protocol.KonferenzraumUser;
import net.javacomm.window.manager.Frames;



/**
 * Ein grafischer Überblick über alle Anwender, die in einem Raum sind.
 * 
 * 
 * @author llange
 *
 */
public class JOnlineOfflinePanel extends JPanel implements Babelfish {

  private static final long serialVersionUID = -1258669005473361153L;
  private final static Logger log = LogManager.getLogger(JOnlineOfflinePanel.class);
  // private final static String names [] = {"", /*"First Name",
  // "Last Name", "UID"*/};
  private TableModel3 model = new TableModel3();
  private JTable table = new JTable();
  private JScrollPane scrollTable = new JScrollPane();
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  private JTableHeader header;
  private ColorRenderer cRenderer = new ColorRenderer();
  private CountUserPanel countUserPanel = new CountUserPanel();
  private ArrayList<KonferenzraumUser> chatuser = new ArrayList<>();
  private HeaderRenderer headerrenderer = new HeaderRenderer();

  public JOnlineOfflinePanel() {
    super(true);
    model.setHeader("");
    setLayout(new BorderLayout());
    table.setModel(model);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // table.setAutoResizeMode (JTable.AUTO_RESIZE_OFF);
    header = table.getTableHeader();
    header.setReorderingAllowed(false);
    header.setDefaultRenderer(headerrenderer);
    headerrenderer.setHorizontalAlignment(SwingConstants.CENTER);

    table.setDefaultRenderer(String.class, cRenderer);
    cRenderer.setHorizontalAlignment(SwingConstants.CENTER);
    table.setRowHeight(28);

    scrollTable.setViewportView(table);
    scrollTable.getHorizontalScrollBar().setUnitIncrement(12);
    scrollTable.getVerticalScrollBar().setUnitIncrement(14);

    countUserPanel.setBorder(null);
    add(BorderLayout.NORTH, countUserPanel);
    add(BorderLayout.CENTER, scrollTable);

    scrollTable.setMinimumSize(new Dimension(84, getSize().height));
    scrollTable.setPreferredSize(new Dimension(84, getSize().height));
    scrollTable.setMaximumSize(new Dimension(84, getSize().height));

    scrollTable.setBorder(null);
    scrollTable.getViewport().setBackground(Resource.JQUERY_TEXTBRAUN);

  }





  public synchronized void addChatInfoListener(PropertyChangeListener l) {
    changes.addPropertyChangeListener(l);
  }





  public synchronized void removeChatInfoListener(PropertyChangeListener l) {
    changes.removePropertyChangeListener(l);
  }



  public void setImage(Frames value) {
    Class<? extends JOnlineOfflinePanel> resource = getClass();
    switch(value) {
      case FORUM:
        headerrenderer.setIcon(new ImageIcon(resource.getResource(Resource.EMBRACE_THE_WORLD)));
        break;
      case PAUSENRAUM:
        headerrenderer.setIcon(new ImageIcon(resource.getResource(Resource.PAUSENRAUM45)));
        break;
      case BESPRECHNUNGSRAUM:
        headerrenderer.setIcon(new ImageIcon(resource.getResource(Resource.BESPRECUNGSRAUM_35x26)));
        break;
      case GRUPPENRAUM:
        headerrenderer.setIcon(new ImageIcon(resource.getResource(Resource.GRUPPENRAUM)));
        break;
      default:
    }

  }



  /**
   * Die Teilnehmerliste wird gelöscht.
   */
  public void clear() {
    model.removeAll();
    chatuser.clear();
    countUserPanel.setCount(chatuser.size());
//    countUserPanel.labelNumber.setText("");
  }


  // TODO

  /**
   * Ein Konferenzteilnehmer wird mit seinem Nickname angezeigt.
   * 
   * 
   * @param user
   */
  public void addUser(KonferenzraumUser user) {
    Util.Record record = new Util.Record();
    Cell<String> cellNickname = new Cell<>();
    cellNickname.setValue(user.getNickname());
    record.add(cellNickname);
    model.addRow(record);
    chatuser.add(user);
    countUserPanel.setCount(chatuser.size());
  }


  // TODO
  // wird relevant, wenn sich der Nickname ändert

  public void updateBackground(String userid, Color color) {
    for (int row = 0; row < model.getRowCount(); row++) {
      String value = model.getValueAt(row, 0).toString();
      if (value.equals(userid)) {

//        chatuser[row].setBackgroundColor(color.getRGB());

        repaint();
        break;
      }
    }
    
    
  }


  // TODO
  // wird relevant, wenn sich der Nickname ändert

  /**
   * Der User hat seine Farbe gewechselt
   * 
   * @param color
   */
  public void updateForeground(String userid, Color color) {
    for (int row = 0; row < model.getRowCount(); row++) {
      String value = model.getValueAt(row, 0).toString();
      if (value.equals(userid)) {
//        chatuser[row].setForegroundColor(color.getRGB());
        repaint();
        break;
      }
    }

  }

  /***************************************************************************/
  /*                                                                         */
  /* Innerclass ColorRenderer */
  /*                                                                         */
  /***************************************************************************/

  class ColorRenderer extends JLabel implements TableCellRenderer {

    private static final long serialVersionUID = 1483352116438614471L;

    public ColorRenderer() {
      setOpaque(true);
    }



    @Override
    public Component getTableCellRendererComponent(JTable table, Object str, boolean isSelected,
        boolean hasFocus, int row, int column) {

      try {
        setBackground(new Color(chatuser.get(row).getBackgroundColor()));
        setForeground(new Color((chatuser.get(row).getForegroundColor())));
        setText((String) str);
      }
      catch (Exception e) {
        log.info(e.getMessage(), e.getCause());
      }
      return this;
    }

  }



  private class HeaderRenderer extends JLabel implements TableCellRenderer {
    private static final long serialVersionUID = 1483352116438614471L;

    public HeaderRenderer() {
      setOpaque(true);
    }



    @Override
    public Component getTableCellRendererComponent(JTable table, Object str, boolean isSelected,
        boolean hasFocus, int row, int column) {

      if (column == 0) {
        setBorder(new EmptyBorder(0, 0, 0, 0));
      }

      setBackground(Resource.JQUERY_ORANGE);

      setText((String) str);
      return this;
    }
  }

  public void setLanguage(KEY key, ISO639 code) {
    countUserPanel.labelAnwesend.setLanguage(key, code);
  }



  @Override
  public void setLanguage(ISO639 code) {
    setLanguage(KEY.LABEL_ANWESEND, code);
  }



  public void setTopBackground(Color color) {
    countUserPanel.setTextBackground(color);
  }



  public void setTopForeground(Color color) {
    countUserPanel.setTextForeground(color);
  }

  static class CountUserPanel extends JPanel {

    private static final long serialVersionUID = 8123825495408532218L;
    private JLabel labelNumber = new JLabel();
    private GridBagLayout gridbag = new GridBagLayout();
    private GridBagConstraints con = new GridBagConstraints();
    private MultilingualLabel labelAnwesend = new MultilingualLabel(KEY.LABEL_ANWESEND);

    public CountUserPanel() {
      setLayout(gridbag);
      setBackground(Resource.JQUERY_ORANGE);

      con.insets = new Insets(2, 2, 0, 2);
      con.gridwidth = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(labelAnwesend, con);
      add(labelAnwesend);
      labelAnwesend.setForeground(Resource.JQUERY_TEXTBRAUN);

      con.insets = new Insets(1, 2, 2, 2);
      gridbag.setConstraints(labelNumber, con);
      labelNumber.setForeground(Resource.JQUERY_TEXTBRAUN);
      add(labelNumber);
      setBorder(null);
    }



    public void setTextBackground(Color color) {
      super.setBackground(color);
    }



    public void setTextForeground(Color color) {
      labelAnwesend.setForeground(color);
      labelNumber.setForeground(color);
    }

    public void setCount(int c) {
      labelNumber.setText(String.valueOf(c));
    }

  }

}


