/**
 * Copyright © 2022, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.screencaster;

import jakarta.ws.rs.client.Client;
import jakarta.ws.rs.client.ClientBuilder;
import jakarta.ws.rs.client.Entity;
import jakarta.ws.rs.core.MediaType;
import java.io.IOException;
import java.util.concurrent.ArrayBlockingQueue;
import org.glassfish.jersey.media.multipart.FormDataMultiPart;
import org.glassfish.jersey.media.multipart.MultiPartFeature;
import net.javacomm.share.Constants;



/**
 * Über den Consumer werden die Screenshots an den Server gesendet.
 * 
 * 
 * @author llange
 *
 */
public class Consumer implements Runnable {

  private ArrayBlockingQueue<byte[]> queue;
  private String userid;
  private boolean notStopped;

  /**
   * Der Consumer wird initialisiert.
   * 
   * @param queue
   *               aus dieser Queue werden die Bilder gelesen.
   * @param userid
   *               an diesen Anwender werden die Bilder gesendet
   */
  public Consumer(ArrayBlockingQueue<byte[]> queue, String userid) {
    this.queue = queue;
    this.userid = userid;
  }



  public void stop() {
    notStopped = false;
  }


  @Override
  public void run() {
    notStopped = true;
    Client client = ClientBuilder.newClient();
    client.register(MultiPartFeature.class);
    while (notStopped) {
      try(FormDataMultiPart formMultipart = new FormDataMultiPart()) {
        byte[] picture = queue.take(); // aus der Queue lesen

//        BufferedImage image = ImageIO.read(new ByteArrayInputStream(picture));
        formMultipart.field("userid", userid);
        formMultipart.field("image", picture, MediaType.APPLICATION_OCTET_STREAM_TYPE);
        
        StringBuilder url = new StringBuilder(Constants.HTTP).append(Constants.DOMAIN)
            .append("/javacommserver/screenshot/save");
        client.target(url.toString()).request(MediaType.APPLICATION_JSON)
            .post(Entity.entity(formMultipart, formMultipart.getMediaType()));
        
      }
      catch (InterruptedException | IOException e) {
        break;
      }
    }
    client.close();
  }

}
