/**
 * Copyright © 2022, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.screencaster;

import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.util.concurrent.ArrayBlockingQueue;
import javax.imageio.ImageIO;



/**
 * Der Producer erzeugt Screenshots im JPG-Format.
 * 
 * 
 * @author llange
 *
 */
public class Producer implements Runnable {

  private Robot robot;
  private Rectangle screenSize;
  private ArrayBlockingQueue<byte[]> queue;
  private boolean notStopped = true;


  /**
   * Der Producer wird initialisiert.
   * 
   * 
   * @param queue
   *                   in diese Queue werden die Screenshots geschrieben
   * @param robot
   * @param screenSize
   *                   die Bildschirmgröße
   */
  public Producer(ArrayBlockingQueue<byte[]> queue, Robot robot, Rectangle screenSize) {
    this.queue = queue;
    this.robot = robot;
    this.screenSize = screenSize;
  }

  

  public void stop() {
    notStopped = false;
  }



  @Override
  public void run() {
    notStopped = true;
    BufferedImage image;
    ByteArrayOutputStream outputStream;
    while (notStopped) {
      image = robot.createScreenCapture(screenSize);
      outputStream = new ByteArrayOutputStream();
      try {
        ImageIO.write(image, "jpg", outputStream);
      }
      catch (Exception e) {}
      try {
        queue.put(outputStream.toByteArray());
      }
      catch (InterruptedException e) {
        break;
      }
    }
  }



}
