package net.javacomm.restserver;

import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.WebApplicationException;
import jakarta.ws.rs.core.MediaType;
import java.io.IOException;
import java.io.InputStream;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.glassfish.jersey.media.multipart.FormDataBodyPart;
import org.glassfish.jersey.media.multipart.FormDataMultiPart;
import org.glassfish.jersey.media.multipart.FormDataParam;
import net.javacomm.database.DatabaseException;
import net.javacomm.database.WebdatabaseImpl;
import net.javacomm.database.entity.PrivateChatfile;
import net.javacomm.transfer.TransferChatfile;



@Path("/chat")
public class ChatFileUpload {

  private final Logger log = LogManager.getLogger(ChatFileUpload.class);

  private WebdatabaseImpl database;

  public ChatFileUpload() {
    database = (WebdatabaseImpl) WebdatabaseImpl.getInstance();
  }



  /**
   * 
   * Der Anwender sendet über den Chat eine Datei an den Server.
   * 
   * 
   * @param number
   *               der Empfänger
   * @param uid
   *               der Absender
   * @param infile
   *               die empfangende Datei als Stream
   * @param meta
   *               Metadaten zur Datei
   * @return {@code true}, die wurde empfangen und gespeichert
   */
  @POST
  @Path("/save")
  @Produces(MediaType.APPLICATION_JSON)
  @Consumes(MediaType.MULTIPART_FORM_DATA)
  public TransferChatfile saveFile(@FormDataParam("number") int number,
      @FormDataParam("absolute") String absoluteFilename, @FormDataParam("file") InputStream infile) {

    boolean done = database.chatfilePhase_2(number, infile);
    PrivateChatfile chatfile = database.fetchChatfile(number);
    TransferChatfile transfer = new TransferChatfile();
    transfer.setError(done);
    transfer.setAbsoluteFilename(absoluteFilename);
    transfer.setReceiverUid(chatfile.getReceiverUid());
    transfer.setUid(chatfile.getUid());
    transfer.setFilesize(chatfile.getFilesize());
    transfer.setMessage(chatfile.getMessage());
    transfer.setFilename(chatfile.getFilename());
    transfer.setNumber(chatfile.getNumber());
    transfer.setRemoteNickname(chatfile.getNickname());
    return transfer;
  }



  @POST
  @Path("/upload")
  public int upload(FormDataMultiPart multiPart) {
    int count = 0;
    FormDataBodyPart anlage = multiPart.getField("anlagennummer");
    FormDataBodyPart chunknummer = multiPart.getField("chunknummer");
    FormDataBodyPart content = multiPart.getField("content");

    try {
      count = database.insertChunk(
          Long.valueOf(anlage.getValue()), Integer.valueOf(chunknummer.getValue()), content.getContent()
      );
    }
    catch (NumberFormatException | IOException | DatabaseException e) {
      log.error(e.getMessage());
      throw new WebApplicationException();
    }
    return count;
  }



  /**
   * Erzeuge einen Anlagenidentifier.
   * 
   * @return dieser Identifier wurde erstellt
   */
  @GET
  @Path("/identifier")
  public long createIdentifier() {
    long identifier;
    try {
      identifier = database.createIdentifier();
    }
    catch (DatabaseException e) {
      log.error(e.getMessage());
      throw new WebApplicationException();
    }
    return identifier;
  }

}
