/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.FlowLayout;
import java.beans.VetoableChangeListener;
import java.beans.VetoableChangeSupport;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.schema.ISO639;



/**
 * Die oberste Spielleiste für die Programmsteuerung und das Spielergebnis.
 */
public class ControlAndResultView extends JPanel implements Babelfish {

  private static final long serialVersionUID = -4094296516011568545L;
  private EtchedBorder etched = new EtchedBorder(EtchedBorder.LOWERED);
  private JPatternResultPanel panPatternResult = new JPatternResultPanel();
  private JControlPanel controlpanel = new JControlPanel();
  private FlowLayout flow = new FlowLayout();

  private VetoableChangeSupport vetos = new VetoableChangeSupport(this);

  public ControlAndResultView() {

    setBackground(Constants.OLIV);
    setLayout(flow);
    add(controlpanel);
    add(panPatternResult);
    setBorder(etched);

    controlpanel.addVetoableControlListener(event -> {
      vetos.fireVetoableChange(event);
    });

  }



  public void setSolutionPattern(String pattern) {
    panPatternResult.showResult(pattern);
  }



  public void addVetoableResultControlListener(VetoableChangeListener listener) {
    vetos.addVetoableChangeListener(listener);
  }



  public void removeVetoableResultControlListener(VetoableChangeListener listener) {
    vetos.removeVetoableChangeListener(listener);
  }



  public void removeAllListener() {
    controlpanel.removeAllListener();
    for (VetoableChangeListener pl : vetos.getVetoableChangeListeners()) {
      removeVetoableResultControlListener(pl);
    }
  }



  /**
   * Die NewGame-Componente kann als Positionsanzeiger für einen Dialog verwendet
   * werden.
   * 
   * @return eine Swing-Componente
   */
  JComponent getComponentNewGame() {
    return controlpanel.getButtonNewGame();
  }



  /**
   * Die StartGame-Componente kann als Positionsanzeiger für einen Dialog
   * verwendet werden.
   * 
   * @return eine Swing-Componente
   */
  JComponent getComponentStartGame() {
    return controlpanel.getButtonStartGame();
  }



  @Override
  public void setLanguage(ISO639 code) {
    controlpanel.setLanguage(code);
    panPatternResult.setLanguage(code);
  }
}
