/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import org.nexuswob.gui.JMouseRadioButton;



/**
 * Der RadioButton blendet einen Handcursor ein, wenn die Maus ihn berührt. Der
 * Knopf hat eine konfigurierbare Vordergrundfarbe und Hintergrundfarbe.
 * 
 */
public class JColorRadioButton extends JMouseRadioButton {

  /**
   * 
   */
  private static final long serialVersionUID = 6740511762883487322L;

  private Color fgColor;
  private Color bgColor;

  /**
   * Ein RadioButton mit einer Vordergrundfarbe und Hintergrundfarbe.
   * 
   * @param fgColor
   *                Vordergrundfarbe
   * @param bgColor
   *                Hintergrundfarbe
   */
  public JColorRadioButton(Color fgColor, Color bgColor) {
    this.fgColor = fgColor;
    this.bgColor = bgColor;
  }



  @Override
  public void paintComponent(Graphics g) {
    Graphics2D g2 = (Graphics2D) g;

    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setColor(bgColor);
    g2.fillRect(0, 0, 50, 50);

    g2.setColor(fgColor);
    if (isSelected()) {
      g2.fill3DRect(5, 5, 40, 40, false);
    }
    else {
      g2.fill3DRect(5, 5, 40, 40, true);
    }
  }



  @Override
  public Dimension getPreferredSize() {
    return new Dimension(50, 50);
  }

}
