/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.beans.VetoableChangeSupport;
import java.io.IOException;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import org.nexuswob.gui.JMouseButton;
import net.javacomm.mastermind.audio.SoundClip;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.MultilingualLabel;
import net.javacomm.multilingual.MultilingualString;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.window.manager.Control;



/**
 * Die Spielknöpfe für die Programmsteuerung.
 */
public class JControlPanel extends JPanel implements Babelfish {

  private static final long serialVersionUID = -5443024729780378774L;

  private MultilingualString newGame = new MultilingualString(KEY.BUTTON_NEUES_SPIEL);
  private JMouseButton buttonNewGame = new JMouseButton(newGame.toString());

  private MultilingualString startGame = new MultilingualString(KEY.BUTTON_SPIEL_STARTEN);
  private JMouseButton buttonStartGame = new JMouseButton(startGame.toString());
  private MultilingualString howToUse = new MultilingualString(KEY.BUTTON_ANLEITUNG);
  private JMouseButton buttonHowToUse = new JMouseButton(howToUse.toString());
  private GridLayout grid = new GridLayout(1, 3, 10, 10);
  private VetoableChangeSupport vetos = new VetoableChangeSupport(this);
  private ImageIcon runningScared = new ImageIcon(getClass().getResource(Constants.GAME_IS_RUNNING));
  private MultilingualLabel labelRunningScared = new MultilingualLabel(KEY.LABEL_SCARED);
  private MultilingualString underConstruction = new MultilingualString(KEY.STRING_UNDER_CONSTRUCTION);
  private ImageIcon iconUnderConstruction = new ImageIcon(
      getClass().getResource(Constants.UNDER_CONSTRUCTION)
  );

  public JControlPanel() {
    setBackground(Constants.OLIV);

    buttonNewGame.setBackground(Constants.OLIV);
    buttonNewGame.setForeground(Constants.WHITE);
    buttonNewGame.addActionListener(event -> {
      try {
        vetos.fireVetoableChange(JControlPanel.class.getName(), Control.NULL, Control.NEW_GAME);
      }
      catch (PropertyVetoException e) {
        // TODO Auto-generated catch block
      }
    });

    buttonStartGame.setBackground(Constants.OLIV);
    buttonStartGame.setForeground(Constants.WHITE);
    buttonStartGame.addActionListener(event -> {
      try {
        vetos.fireVetoableChange(JControlPanel.class.getName(), Control.NULL, Control.START_GAME);
      }
      catch (PropertyVetoException e) {
        try {
          SoundClip.playWhistleGame();
        }
        catch (IOException | UnsupportedAudioFileException | LineUnavailableException e1) {}
        JOptionPane.showInternalMessageDialog(
            buttonStartGame, labelRunningScared, startGame.toString(), JOptionPane.PLAIN_MESSAGE,
            runningScared
        );
      }
    });

    buttonHowToUse.setBackground(Constants.OLIV);
    buttonHowToUse.setForeground(Constants.WHITE);
    buttonHowToUse.addActionListener(event -> {
      try {
        SoundClip.playElectric();
      }
      catch (IOException | UnsupportedAudioFileException | LineUnavailableException e) {}
      JOptionPane.showInternalMessageDialog(
          this, iconUnderConstruction, underConstruction.toString(), JOptionPane.PLAIN_MESSAGE
      );
      SoundClip.stopElectric();

    });

    setLayout(grid);
    this.add(buttonHowToUse);
    this.add(buttonNewGame);
    this.add(buttonStartGame);
  }



  public void addVetoableControlListener(VetoableChangeListener listener) {
    vetos.addVetoableChangeListener(listener);
  }



  public void removeVetoableControlListener(VetoableChangeListener listener) {
    vetos.removeVetoableChangeListener(listener);
  }



  public void removeAllListener() {
    for (ActionListener al : buttonHowToUse.getActionListeners()) {
      buttonHowToUse.removeActionListener(al);
    }
    for (ActionListener al : buttonNewGame.getActionListeners()) {
      buttonNewGame.removeActionListener(al);
    }
    for (ActionListener al : buttonStartGame.getActionListeners()) {
      buttonStartGame.removeActionListener(al);
    }
    for (VetoableChangeListener vl : vetos.getVetoableChangeListeners()) {
      removeVetoableControlListener(vl);
    }
  }



  @Override
  public void setLanguage(ISO639 code) {
    howToUse.setLanguage(code);
    buttonHowToUse.setText(howToUse.toString());

    startGame.setLanguage(code);
    buttonStartGame.setText(startGame.toString());

    newGame.setLanguage(code);
    buttonNewGame.setText(newGame.toString());

    labelRunningScared.setLanguage(code);
    underConstruction.setLanguage(code);
  }



  /**
   * Der Button kann als Positionsanzeiger für einen Dialog verwendet werden.
   * 
   * @return ein JButton
   */
  JButton getButtonNewGame() {
    return buttonNewGame;
  }



  /**
   * Der Button kann als Positionsanzeiger für einen Dialog verwendet werden.
   * 
   * @return dieser {@link #buttonStartGame}
   */
  JButton getButtonStartGame() {
    return buttonStartGame;
  }

}
