/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import net.javacomm.mastermind.tool.DefColors;
import net.javacomm.mastermind.tool.Knopf;



@SuppressWarnings("serial")
public class JPatternResultPaintPanel extends JPanel {

  private EtchedBorder etched = new EtchedBorder(EtchedBorder.LOWERED);
  private JPaintedFigurePanel[] figures = new JPaintedFigurePanel[Knopf.values().length];
  private GridBagLayout gridbag = new GridBagLayout();
  private GridBagConstraints con = new GridBagConstraints();
//  private String resultCode = String.valueOf(
//      new char[] {DefColors.GRAY.charValue(), DefColors.GRAY.charValue(), DefColors.GRAY.charValue(),
//          DefColors.GRAY.charValue()}
//  );

  public JPatternResultPaintPanel() {
    setBackground(Constants.OLIV);

    setLayout(gridbag);

    con.gridy = 0;
    con.weightx = 1.0;

    for (int index = 0; index < figures.length; index++) {
      figures[index] = new JPaintedFigurePanel(DefColors.GRAY.colorValue());
      con.gridx = index;
      con.insets = new Insets(5, 5, 5, 0);
      gridbag.setConstraints(figures[index], con);
      add(figures[index]);
    }
    setBorder(etched);
  }

//  /**
//   * Das Lösungsmuster wird übergeben.
//   * 
//   * @param resultCode
//   *                   das Lösungsmuster
//   */
//  public void setResultCode(String resultCode) {
//    this.resultCode = resultCode;
//  }



  /**
   * Das Lösungsmuster wird angezeigt.
   */
  public void showResult(String resultCode) {
    for (int index = 0; index < resultCode.length(); index++) {
      figures[index].setFigureColor(DefColors.toDefColors(resultCode.charAt(index)).colorValue());
    }
  }

}
