/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui.manual;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import net.javacomm.mastermind.gui.Constants;

public class JManualFrame extends JFrame implements PropertyChangeListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4762012017155729935L;
	public static final int MANUAL_COUNT = 7;

	private JManualPanel[] panManual;
	private String[] strPanels = { Constants.MANUAL1, Constants.MANUAL2, Constants.MANUAL3, Constants.MANUAL4,
			Constants.MANUAL5, Constants.MANUAL6, Constants.MANUAL7 };
	private int index = 0;
	private JManualControlPanel panControl;
	private JPanel mainPanel;
	private JPanel panCard;
	private BorderLayout border = new BorderLayout();
	private CardLayout card = new CardLayout();
	private Toolkit toolkit;
	private WindowAction actionWindow = new WindowAction();

	public JManualFrame(Image[] images) {

		this.addWindowListener(actionWindow);
		panControl = new JManualControlPanel();
		panControl.addPropertyChangeListener(this);

		panCard = new JPanel();
		panCard.setLayout(card);

		mainPanel = new JPanel();
		mainPanel.setLayout(border);
		mainPanel.add(panCard, BorderLayout.CENTER);
		mainPanel.add(panControl, BorderLayout.SOUTH);

		panManual = new JManualPanel[MANUAL_COUNT];

		for (int i = 0; i < MANUAL_COUNT; i++) {
			panManual[i] = new JManualPanel(images[i]);
			panCard.add(panManual[i], strPanels[i]);
		}

		card.show(panCard, strPanels[index]);

		// Frameparameter
		this.setContentPane(mainPanel);

		toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.setBounds(0, 0, screenSize.width, screenSize.height);
		System.out.println("Bildschirmbreite: "+screenSize.width+"; Bildschirmh�he: "+screenSize.height);
		this.setResizable(false);
		this.setVisible(true);
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		String actionCommand = event.getPropertyName();
		if(actionCommand.equals(Constants.BUTTON_NEXT)) {
			if(index < (MANUAL_COUNT-1)) {
				index++;
				card.show(panCard, strPanels[index]);
			}
		}
		if(actionCommand.equals(Constants.BUTTON_BACK)){
			if(index > 0) {
				index--;
				card.show(panCard, strPanels[index]);
			}
		}
		if (actionCommand.equals(Constants.BUTTON_QUIT)) {
			closeWindow();
		}
		
	}

	public void closeWindow() {
		this.setVisible(false);
		this.dispose();
	}

	private class WindowAction extends WindowAdapter {

		@Override
		public void windowClosing(WindowEvent event) {
			closeWindow();
		}
	}
}
