/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.tool;

public enum Knopf {

  Knopf_1(0),
  Knopf_2(1),
  Knopf_3(2),
  Knopf_4(3);

  private int index;

  private Knopf(int index) {
    this.index = index;
  }



  public int intValue() {
    return index;
  }



  /**
   * Konvertiere von {@code int} nach {@code Knopf}.
   * 
   * @param value
   *              index aus dem Intervall [0..3]
   * 
   * @return {@link #Knopf_1}, {@link #Knopf_2}, {@link #Knopf_3} oder
   *         {@link #Knopf_4}
   */
  public static Knopf toKnopf(int value) {

    for (Knopf tmp : Knopf.values()) {
      if (tmp.intValue() == value) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }

}
