/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;



public class ChatMessageSerializer implements JsonSerializer<CHATMESSAGE> {

  @Override
  public JsonElement serialize(CHATMESSAGE src, Type typeOfSrc, JsonSerializationContext context) {

    JsonObject jsonObject = new JsonObject();
    
    JsonObject dataset = new JsonObject();
    dataset.add("ROOM", context.serialize(src.getRoom()));
    dataset.add("DATETIME", context.serialize(src.getDatetime()));
    dataset.add("MESSAGE", context.serialize(src.getMessage()));
    dataset.add("ATTACHMENT", context.serialize(src.getAttachment()));
    dataset.add("FILENAME", context.serialize(src.getFilename()));
    dataset.add("FILESIZE", context.serialize(src.getFilesize()));
    dataset.add("CHATUSER", context.serialize(src.getChatUser()));

    jsonObject.add("HEADER", context.serialize(src.getHeader()));
    jsonObject.add("COMMAND", context.serialize(src.getCommand()));
    jsonObject.add("DATASET", context.serialize(dataset));

    return jsonObject;

  }

}
