/**
 * Copyright © 2020 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import java.util.regex.Pattern;



/**
 * 
 * Das Applikationsprotokoll zwischen Controller und Portal hat das Format
 * HEADER | COMMAND | DATASET. Das Protokoll sitzt oberhalb von <i>JSON</i>. In
 * der Klasse sind alle Schlüsselwörter als Konstante für die Datenübertragung
 * hinterlegt. Spätere Neuerungen oder Änderungen sind über diese Klasse
 * vorzunehmen.
 * 
 * @author llange
 *
 */
public abstract class Protocol {

  final static Pattern PATTERN_JSON = Pattern.compile("\\{\"HEADER\".+\"COMMAND\".+\\}");
  /**
   * Splittet einen Dateipfadnamen in Pfad und Name auf.
   */
  public final static Pattern PATH_SEPARATOR = Pattern.compile("(.*[\\x5c/])(.+)$");

  /**
   * Ein JSON Key
   */
  public final static String HEADER = "HEADER";
  /**
   * Ein JSON Key
   */
  public final static String DATASET = "DATASET";

}
