/**
 * Copyright © 2020 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

/**
 * Ein neuer User meldet sich an.
 * 
 * @author llange
 *
 */
public class SIGNIN extends USER {



  @Override
  public String toString() {
    GsonBuilder builder = new GsonBuilder().disableHtmlEscaping();
    Gson gson = builder.registerTypeAdapter(SIGNIN.class, new SigninSerializer()).create();
    return gson.toJson(this);
  }

  public enum Benutzerstatus {

    VOLLMITGLIED(1),
    MÖCHTE_VOLLMITGLIED_WERDEN(2),
    VOLLMITGLIEDSCHAFT_BESTÄTIGT(3),
    MITGLIEDSCHAFT_ABGELEHNT(4);

    private int status;

    private Benutzerstatus(int value) {
      status = value;
    }



    @Override
    public String toString() {
      return name();
    }



    public int getStatus() {
      return status;
    }



    public static Benutzerstatus toBenutzerstatus(int value) {

      for (Benutzerstatus tmp : Benutzerstatus.values()) {
        if (tmp.getStatus() == value) {
          return tmp;
        }
      }
      throw new IllegalArgumentException();
    }

  }

}
