/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import java.lang.reflect.Type;
import net.javacomm.multilingual.schema.KEY;



public class SigninDeserializer implements JsonDeserializer<SIGNIN> {

  @Override
  public SIGNIN deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {

    JsonObject jsonobject = json.getAsJsonObject();
    HEADER header = HEADER.toHeader(jsonobject.getAsJsonPrimitive(Protocol.HEADER).getAsString());
    JsonObject dataset = jsonobject.getAsJsonObject(Protocol.DATASET);
    String email = dataset.get(SIGNIN.EMAIL).getAsString();
    String password = null;
    try {
      password = dataset.get(SIGNIN.PASSWORD).getAsString();
    }
    catch (NullPointerException e) {}
    String userid = null;
    try {
      userid = dataset.get(SIGNIN.USERID).getAsString();
    }
    catch (NullPointerException e) {}
    String nickname = null;
    try {
      nickname = dataset.get(SIGNIN.NICKNAME).getAsString();
    }
    catch (NullPointerException e) {}
    String ip = null;
    try {
      ip = dataset.get(SIGNIN.IP).getAsString();
    }
    catch (NullPointerException e) {}
    Integer fgcolor = null;
    try {
      fgcolor = dataset.get(SIGNIN.FOREGROUND_COLOR).getAsInt();
    }
    catch (NullPointerException e) {}
    Integer bgcolor = null;
    try {
      bgcolor = dataset.get(SIGNIN.BACKGROUND_COLOR).getAsInt();
    }
    catch (NullPointerException e) {}
    KEY multilingualkey = null;
    try {
      multilingualkey = KEY.fromValue(dataset.getAsJsonPrimitive(SIGNIN.MULTILINGUALKEY).getAsString());
    }
    catch (NullPointerException e) {}
    String text = null;
    try {
      text = dataset.get(SIGNIN.TEXT).getAsString();
    }
    catch (NullPointerException e) {}
    Integer errorcode = null;
    try {
      errorcode = dataset.get(SIGNIN.ERRORCODE).getAsInt();
    }
    catch (NullPointerException e) {}
    String errormessage = null;
    try {
      errormessage = dataset.get(SIGNIN.ERRORMESSAGE).getAsString();
    }
    catch (NullPointerException e) {}
    String neuesPassword = null;
    try {
      neuesPassword = dataset.get(SIGNIN.NEUES_PASSWORD).getAsString();
    }
    catch (NullPointerException e) {}
    String confirmationKey = null;
    try {
      confirmationKey = dataset.get(SIGNIN.CONFIRMATION_KEY).getAsString();
    }
    catch (NullPointerException e) {}

    SIGNIN signin = new SIGNIN();
    signin.setHeader(header);
    signin.setCommand(Command.SIGNIN);
    signin.setDataset(Protocol.DATASET);
    signin.setEmail(email);
    signin.setPassword(password);
    signin.setUserid(userid);
    signin.setNickname(nickname);
    signin.setIp(ip);
    signin.setForegroundColor(fgcolor);
    signin.setBackgroundColor(bgcolor);
    signin.setMultilingualkey(multilingualkey);
    signin.setText(text);
    signin.setErrorCode(errorcode);
    signin.setErrorMessage(errormessage);
    signin.setNeuesPassword(neuesPassword);
    signin.setConfirmationKey(confirmationKey);

    return signin;

  }

}
