/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;

public class SigninSerializer implements JsonSerializer<SIGNIN> {

  @Override
  public JsonElement serialize(SIGNIN src, Type typeOfSrc, JsonSerializationContext context) {

    JsonObject jsonObject = new JsonObject();

    JsonObject dataset = new JsonObject();
    dataset.add("EMAIL", context.serialize(src.getEmail()));
    dataset.add("USERID", context.serialize(src.getUserid()));
    dataset.add("IP", context.serialize(src.getIp()));
    dataset.add("PASSWORD", context.serialize(src.getPassword()));
    if (src.getForegroundColor() != null)
      dataset.add("FOREGROUND_COLOR", context.serialize(src.getForegroundColor()));
    if (src.getBackgroundColor() != null)
      dataset.add("BACKGROUND_COLOR", context.serialize(src.getBackgroundColor()));
    dataset.add("NICKNAME", context.serialize(src.getNickname()));

    if (src.getErrorCode() != null) dataset.add("ERRORCODE", context.serialize(src.getErrorCode()));
    dataset.add("ERRORMESSAGE", context.serialize(src.getErrorMessage()));

    if (src.getText() != null) dataset.add("TEXT", context.serialize(src.getText()));

    if (src.getMultilingualkey() != null)
      dataset.add("MULTILINGUALKEY", context.serialize(src.getMultilingualkey().value()));

    jsonObject.add("HEADER", context.serialize(src.getHeader()));
    jsonObject.add("COMMAND", context.serialize(src.getCommand()));
    jsonObject.add("DATASET", context.serialize(dataset));

    return jsonObject;

  }

}
