/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import static net.javacomm.protocol.HEADER.ERROR;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;



public class UsrloginSerializer implements JsonSerializer<USRLOGIN> {

  @Override
  public JsonElement serialize(USRLOGIN src, Type typeOfSrc, JsonSerializationContext context) {
    JsonObject jsonObject = new JsonObject();
    JsonObject dataset = new JsonObject();
    dataset.add("EMAIL", context.serialize(src.getEmail()));
    dataset.add("USERID", context.serialize(src.getUserid()));
    dataset.add("PASSWORD", context.serialize(src.getPassword()));
    dataset.add("IP", context.serialize(src.getIp()));
    dataset.add("FOREGROUND_COLOR", context.serialize(src.getForegroundColor()));
    dataset.add("BACKGROUND_COLOR", context.serialize(src.getBackgroundColor()));
    dataset.add("VOLUME", context.serialize(src.getVolume()));
    dataset.add("ONCALL", context.serialize(src.isOncall()));
    dataset.add("NICKNAME", context.serialize(src.getNickname()));

    jsonObject.add("HEADER", context.serialize(src.getHeader()));
    jsonObject.add("COMMAND", context.serialize(src.getCommand()));
    if (ERROR == (src.getHeader())) return jsonObject;
    jsonObject.add("DATASET", context.serialize(dataset));
    return jsonObject;
  }

}
