/*
 * The MIT License
 * Copyright © 2025 Luis Andrés Lange <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.restserver;

import java.io.IOException;
import java.io.InputStream;



/**
 * Dieser InputStream zählt die Anzahl der gelesenen Bytes.
 */
public class CountingInputStream extends InputStream {

  private final InputStream delegate;
  private long bytesRead;

  public CountingInputStream(InputStream delegate) {
    this.delegate = delegate;
    this.bytesRead = 0;
  }



  @Override
  public int read() throws IOException {
    int byteRead = delegate.read();
    if (byteRead != -1) {
      bytesRead++;
    }
    return byteRead;
  }



  @Override
  public int read(byte[] b, int off, int len) throws IOException {
    int bytesReadNow = delegate.read(b, off, len);
    if (bytesReadNow != -1) {
      bytesRead += bytesReadNow;
    }
    return bytesReadNow;
  }



  public long getBytesRead() {
    return bytesRead;
  }
}
