/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.restserver;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.WebApplicationException;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.MultivaluedMap;
import jakarta.ws.rs.ext.MessageBodyReader;
import jakarta.ws.rs.ext.MessageBodyWriter;
import jakarta.ws.rs.ext.Provider;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import net.javacomm.protocol.SEARCHFILES;



@Provider
@Produces(MediaType.APPLICATION_JSON)
public class MessageBodySEARCHFILES
    implements MessageBodyWriter<SEARCHFILES>, MessageBodyReader<SEARCHFILES> {

  @Override
  public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
    return type == SEARCHFILES.class;
  }



  @Override
  public void writeTo(SEARCHFILES searchfiles, Class<?> type, Type genericType, Annotation[] annotations,
      MediaType mediaType, MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream)
      throws IOException, WebApplicationException {

    Gson gson = new Gson();
    try(OutputStreamWriter writer = new OutputStreamWriter(entityStream, "UTF-8")) {
      gson.toJson(searchfiles, writer);
    }

  }



  @Override
  public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
    return type == SEARCHFILES.class;
  }



  @Override
  public SEARCHFILES readFrom(Class<SEARCHFILES> type, Type genericType, Annotation[] annotations,
      MediaType mediaType, MultivaluedMap<String, String> httpHeaders, InputStream entityStream)
      throws IOException, WebApplicationException {

    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    try(BufferedReader buffer = new BufferedReader(new InputStreamReader(entityStream, "UTF-8"))) {
      return gson.fromJson(buffer, type);
    }

  }

}
