/**
 * Copyright © 2021 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.restserver;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.lang.reflect.Type;
import java.util.List;
import net.javacomm.transfer.TransferOutdated;
import net.javacomm.transfer.TransferRoomfilter;
import net.javacomm.transfer.TransferUser;



/**
 * Der Wrapper ist eine Hilfsklasse für das Übertragen von Listen jeglichen
 * Typs.
 * 
 * @author llange
 *
 */
public class Wrapper {

  private String liste;

  private Wrapper(String value) {
    liste = value;
  }



  public static Wrapper fromString(String value) {
    return new Wrapper(value);
  }


  
  /**
   * Ein Transfer-REQUEST-Parameter für den Consumer. Der Paramter wird auf der
   * Serverseite ausgewertet.
   * 
   * @return eine Liste von Transferobjekte
   */
  public List<TransferUser> getTransferUser() {
    Gson gson = new Gson();
    Type typeOfT = new TypeToken<List<TransferUser>>() {}.getType();
    return gson.fromJson(liste, typeOfT);
  }


  
  /**
   * Ein Transfer-REQUEST-Parameter für den Consumer. Der Paramter wird auf der
   * Serverseite ausgewertet.
   * 
   * @return eine Liste von Transferobjekte
   */
  public List<TransferRoomfilter> getTransferRoomfilter() {
    Gson gson = new Gson();
    Type typeOfT = new TypeToken<List<TransferRoomfilter>>() {}.getType();
    return gson.fromJson(liste, typeOfT);
  }



  /**
   * Ein Transfer-REQUEST-Parameter für den Consumer. Der Parameter wird auf der
   * Serverseite ausgewertet.
   * 
   * @return eine Liste von Transferobjekte
   */
  public List<TransferOutdated> getTransferOutdated() {
    Gson gson = new Gson();
    Type typeOfT = new TypeToken<List<TransferOutdated>>() {}.getType();
    return gson.fromJson(liste, typeOfT);
  }



  /**
   * Ein Transfer-REQUEST-Parameter für den Consumer. Der Parameter wird auf der
   * Serverseite ausgewertet.
   * 
   * @return eine Liste von Transferobjekte
   */
  public List<String> getStringList() {
    Gson gson = new Gson();
    Type typeOfT = new TypeToken<List<String>>() {}.getType();
    return gson.fromJson(liste, typeOfT);
  }



  /**
   * Für das Einkapseln von Listen in einen JSON-String.
   * 
   * @param <T>
   *              der Listentyp
   * @param liste
   *              diese Liste wird geschickt
   * @return die gewandelte Liste als JSON-String
   */
  public static <T> String toString(List<T> liste) {
    Gson gson = new Gson();
    return gson.toJson(liste);
  }

}
