/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.transfer;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.time.ZonedDateTime;
import java.util.TreeMap;



/**
 * Das Transferobjekt übertägt die Maskeninhalte von Telefonkonferenz
 * organisieren.
 *
 * @author lromero
 *
 */
public class TransferOrganizeConferenceCall {

  private String konferenzname; // Konferenzname
  private String beschreibung;
  private TreeMap<String, String> teilnehmer; // alle Teilnehmer (Nickname, UID)
  private ZonedDateTime zonedAblaufdatum;

  public String getKonferenzname() {
    return konferenzname;
  }



  public void setKonferenzname(String konferenzname) {
    this.konferenzname = konferenzname;
  }



  public String getBeschreibung() {
    return beschreibung;
  }



  public void setBeschreibung(String beschreibung) {
    this.beschreibung = beschreibung;
  }



  /**
   *
   * @return n-Paare (Nickname/Userid)
   */
  public TreeMap<String, String> getTeilnehmer() {
    return teilnehmer;
  }



  /**
   * Alle Konferenzteilnehmer werden übergeben.
   *
   * @param teilnehmer
   *                   alle Teilnehmer
   */
  public void setTeilnehmer(TreeMap<String, String> teilnehmer) {
    this.teilnehmer = teilnehmer;
  }




  @Override
  public String toString() {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.registerTypeAdapter(ZonedDateTime.class, new ZonedDateTimeSerializer()).create();
    return gson.toJson(this);
  }



  public ZonedDateTime getZonedAblaufdatum() {
    return zonedAblaufdatum;
  }



  public void setZonedAblaufdatum(ZonedDateTime zonedAblaufdatum) {
    this.zonedAblaufdatum = zonedAblaufdatum;
  }

}
