/*
 * Copyright © 2022, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.window.manager;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.border.EmptyBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.Multilingual;
import net.javacomm.multilingual.schema.COMMAND;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.multilingual.schema.LANGUAGE;
import net.javacomm.multilingual.schema.LANGUAGES;



/*****************************************************************************/
/*                                                                           */
/*                                Class WindowManager                        */
/*                                                                           */
/*****************************************************************************/

@SuppressWarnings("serial")
public class WindowManager extends JPanel implements Babelfish {

  private final static Logger log = LogManager.getLogger(WindowManager.class);
  private DefaultMutableTreeNode root = new DefaultMutableTreeNode();
  private DefaultTreeModel model = new DefaultTreeModel(root);
  private JScrollPane scroll = new JScrollPane();
  private JTree tree = new JTree();
  private Renderer renderer = new Renderer();
  private Object nodeSelected = new Object();
  private DefaultMutableTreeNode treeNode;
  private Class<? extends WindowManager> resource;
  private ImageIcon launchIcon;
  private ImageIcon windowList;
  private ImageIcon windowNew;
  private ImageIcon infoIcon;
  private ImageIcon privatetalkIcon;
  private ImageIcon ftpIcon;
  private ImageIcon phoneIcon;
  private ImageIcon playerIcon;
  private ImageIcon configureIcon;
  private ImageIcon callIcon;
  private ImageIcon incomingcallIcon;
  private ImageIcon forumIcon;
  private ImageIcon stammtischIcon;
  private ImageIcon pausenraumIcon;
  private ImageIcon schwarzeListeIcon;
  private ImageIcon administratorIcon;
  private ImageIcon screencastIcon;
  private ImageIcon screencastReceiverIcon;
  private ImageIcon projektorIcon;
  private ImageIcon tvIcon;
  private ImageIcon gruppenraumIcon;
  private ImageIcon telefonkonferenzIcon;
  private ImageIcon odxIcon;
  private ImageIcon odxreportIcon;
  private ImageIcon odxMergeIcon;
  private ImageIcon odxValidatorIcon;
  private ImageIcon mastermindIcon;
  private DefaultMutableTreeNode node;
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  private TreeSelectionModel selectionModel;
  private Vector<WM> internalFramelist = new Vector<WM>(6, 6);
  private boolean offline = false;
  private String record;
  private ISO639 language;
  private KEY key;
  private int metricwidth;

  public WindowManager() {
    language = ISO639.DE;
    setLayout(new BorderLayout());
    resource = getClass();


    launchIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.LAUNCH));
    windowList = new ImageIcon(resource.getClassLoader().getResource(WMResource.WINDOW_LIST));
    windowNew = new ImageIcon(resource.getClassLoader().getResource(WMResource.WINDOW_NEW));
    infoIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.INFO_25x25));
    privatetalkIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.PRIVAT_26x26));
    ftpIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.FTP));
    phoneIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.TELEFONEINRICHTEN_42x26));
    playerIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.MP3PLAYER));
    configureIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.CONFIGURE));
    callIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.CALL));
    incomingcallIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.PHONE_RING));
    forumIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.FORUM_32x26));
    stammtischIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.BESPRECUNGSRAUM_35x26));
    pausenraumIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.PAUSENRAUM));
    gruppenraumIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.GRUPPENRAUM_25x26));
    schwarzeListeIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.SCHWARZE_LISTE));
    administratorIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.ADMINISTRATOR));
    screencastIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.SHARE_36x26));
    screencastReceiverIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.HANDSHAKE_39x26));
    projektorIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.PROJEKTOR_43x26));
    tvIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.TVTEST_41x26));
    telefonkonferenzIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.CONFERENCE_23x26));
    odxIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.ODX));
    odxreportIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.ODXREPORT));
    odxMergeIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.ODXMERGE_25x26));
    odxValidatorIcon = new ImageIcon(
        resource.getClassLoader().getResource(WMResource.ODX_SCHEMA_VALIDIEREN_19x26)
    );
    mastermindIcon = new ImageIcon(resource.getClassLoader().getResource(WMResource.MASTERMIND_19x26));



    // root.setUserObject ("Fenster hervorheben");

    // tree.setExpandsSelectedPaths (true);
    tree.setToggleClickCount(1);
    tree.setModel(model);
    tree.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

    tree.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent event) {
        TreePath path = tree.getPathForLocation(event.getX(), event.getY());
        if (path == null) {
          selectionModel.clearSelection();
          return;
        }
        if (path.getParentPath() == null) {
          selectionModel.clearSelection();
          return;
        }
        nodeSelected = path.getLastPathComponent();
        treeNode = (DefaultMutableTreeNode) nodeSelected;
        selectionModel.clearSelection();
        changes.firePropertyChange(treeNode.toString(), Control.NULL, Control.SELECTED);
      }
    });

    tree.setCellRenderer(renderer);
    selectionModel = tree.getSelectionModel();
    scroll.setViewportView(tree);
    scroll.setBorder(null);
    scroll.getHorizontalScrollBar().setUnitIncrement(12);
    scroll.getVerticalScrollBar().setUnitIncrement(12);

    add(BorderLayout.CENTER, scroll);


  }



  /**
   * Der alte Fenstername wird durch den neuen ersetzt.
   *
   * @param oldname
   *                nombre de la ventana antigua
   * @param newname
   *                nombre de la ventana nueva
   */
  public void replaceFramename(String oldname, String newname) {
    if (oldname == null) throw new IllegalArgumentException("oldname ist null");
    if (oldname == "") return;
    if (oldname.equals(newname)) return;
    Enumeration<TreeNode> enum15 = root.preorderEnumeration();
    for (; enum15.hasMoreElements();) {
      node = (DefaultMutableTreeNode) enum15.nextElement();
      if (node.toString().equals(oldname)) {
        FontMetrics fm = getFontMetrics(getFont());
        metricwidth = fm.stringWidth(newname);
        node.setUserObject(newname);
        model.nodeChanged(node);
        break;
      }
    }
  }




  /**
   * Hänge ein neues Fenster in die Baumansicht.
   *
   * @param wm
   *            ein InternalFrame
   * @param der
   *            der Blattname in der Baumansicht sollte der Fenstertitel sein
   */
  public void addWindow(WM wm) {
    // Ist WM in Internal List
    for (WM tmp : internalFramelist) {
      if (tmp.getId() == wm.getId()) return;
    }
    internalFramelist.addElement(wm);
    // hier wird der Knotenname gesetzt
    node = new DefaultMutableTreeNode(wm);
    FontMetrics fm = getFontMetrics(getFont());
    metricwidth = fm.stringWidth(wm.getBaumansicht());

    node.setUserObject(wm.getBaumansicht());
    model.insertNodeInto(node, root, 0);

    tree.scrollPathToVisible(new TreePath(node.getPath()));
    scroll.getHorizontalScrollBar().setValue(0);
    record = wm.getBaumansicht();
  }






  public void addWMListener(PropertyChangeListener l) {
    changes.addPropertyChangeListener(l);
  }



  // muss erhalten bleiben wegen Private Chat
  public void offline() {
    tree.setBackground(WMResource.JQUERY_RED);
    offline = true;
  }



  public void removeAllListener() {

    for (PropertyChangeListener tmp : changes.getPropertyChangeListeners()) {
      removeWMListener(tmp);
    }

    for (MouseListener tmp : tree.getMouseListeners()) {
      tree.removeMouseListener(tmp);
    }


  }



  public void removeWM(WM wm) {
    Enumeration<TreeNode> enum15 = root.preorderEnumeration();
    for (; enum15.hasMoreElements();) {
      node = (DefaultMutableTreeNode) enum15.nextElement();
      if (node.toString().equals(wm.getBaumansicht())) {
        model.removeNodeFromParent(node);
        internalFramelist.removeElement(wm);
        break;
      }
    }
  }



  /**
   * Eine Liste aller Titelfenster.
   *
   * @return
   */
  public List<String> getFrameList() {
    List<String> titel = new ArrayList<>();
    for (WM wm : internalFramelist) {
      titel.add(wm.getFrameTitleId());
    }
    return titel;
  }



  public void removeWMListener(PropertyChangeListener l) {
    changes.removePropertyChangeListener(l);
  }



  /**
   * In der Navigationsansicht wird das aktuelle Fenster farblich hervorgehoben.
   *
   * @param value
   *              der Fenstername in der Baumansicht
   */
  public void setActivNode(String value) {
    record = value;
  }

  /***************************************************************************/
  /*                                                                         */
  /* Innerclass Renderer */
  /*                                                                         */
  /***************************************************************************/

  class Renderer extends DefaultTreeCellRenderer {

    private static final long serialVersionUID = -4795184497920125981L;

    public Renderer() {
      setOpaque(false);
    }



    @Override
    public Dimension getPreferredSize() {
      int width = Math.max(WindowManager.this.getSize().width, metricwidth + 70);
      return new Dimension(width, 32);
    }


    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded,
        boolean leaf, int row, boolean hasFocus) {

      if (offline) {
        setBackground(WMResource.JQUERY_RED);
      }
      else {
        setBackground(WMResource.JQUERY_ORANGE);
      }
      setText(value.toString());

      if (value.toString().equals(record)) {
        setTextSelectionColor(WMResource.JQUERY_BLAU);
        setTextNonSelectionColor(WMResource.JQUERY_BLAU);
      }
      else {
        setTextSelectionColor(WMResource.JQUERY_TEXTBRAUN);
        setTextNonSelectionColor(WMResource.JQUERY_TEXTBRAUN);
      }

      if (value.equals(root)) {
        if (tree.isExpanded(0)) {
          setOpenIcon(windowList);
        }
        else {
          setClosedIcon(windowNew);
        }
      }
      else {
        Frames type;
        for (WM internalFrame : internalFramelist) {
          type = internalFrame.getType();
          String txt = internalFrame.getBaumansicht();
          if (value.toString().equals(txt)) {
            switch(type) {
              case ODX_SCHEMA_VALIDATOR:
                setLeafIcon(odxValidatorIcon);
                continue;
              case ODX_MERGE_EV:
                setLeafIcon(odxMergeIcon);
                continue;
              case ODX_REPORT:
                setLeafIcon(odxreportIcon);
                continue;
              case ODX:
                setLeafIcon(odxIcon);
                continue;
              case INFO:
                setLeafIcon(infoIcon);
                continue;
              case LAUNCHER:
                setLeafIcon(launchIcon);
                continue;
              case FORUM:
                setLeafIcon(forumIcon);
                continue;
              case BESPRECHNUNGSRAUM:
                setLeafIcon(stammtischIcon);
                continue;
              case GRUPPENRAUM:
                setLeafIcon(gruppenraumIcon);
                continue;
              case PAUSENRAUM:
                setLeafIcon(pausenraumIcon);
                continue;
              case PRIVATE_CHAT:
                setLeafIcon(privatetalkIcon);
                continue;
              case FTS:
                setLeafIcon(ftpIcon);
                continue;
              case TELEFON_EINRICHTEN:
                setLeafIcon(phoneIcon);
                continue;
              case PLAYER:
                setLeafIcon(playerIcon);
                continue;
              case CONFIGURE:
                setLeafIcon(configureIcon);
                continue;
              case ANRUFEN:
                setLeafIcon(callIcon);
                continue;
              case ABNEHMEN:
                setLeafIcon(incomingcallIcon);
                continue;
              case SCHWARZE_LISTE:
                setLeafIcon(schwarzeListeIcon);
                continue;
              case ADMINISTRATOR:
                setLeafIcon(administratorIcon);
                continue;
              case VIDEOREKORDER:
                setLeafIcon(screencastIcon);
                continue;
              case SCREENRECEIVER:
                setLeafIcon(screencastReceiverIcon);
                continue;
              case PROJEKTOR:
                setLeafIcon(projektorIcon);
                continue;
              case TV:
                setLeafIcon(tvIcon);
                continue;
              case TELEFONKONFERENZ:
                setLeafIcon(telefonkonferenzIcon);
                continue;
              case MASTERMIND:
                setLeafIcon(mastermindIcon);
                continue;
              default:
                log.warn("unknown roomtype - " + value);
            }
          }
        }
      }
      super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
      setBorder(new EmptyBorder(3, 3, 3, 3));
      return (this);
    }
  }

  /**
   * Der Menuschlüssel für die Textanzeige wird gesetzt.
   *
   * @param key
   *            der Zugriffschlüssel für JMenu, JMenuItem und JCheckBoxMenuItem in
   *            der Oberfläche
   *
   */
  public void setKey(KEY key) {
    setLanguage(key, language);
  }



  /**
   * Die Sprache wird gesetzt.
   *
   * @param language
   *                 die Sprache
   */
  @Override
  public void setLanguage(ISO639 language) {
    setLanguage(this.key, language);
  }



  /**
   * Die Sprache darf jederzeit geändert werden.
   *
   * @param key
   *                 der Schlüssel für das ausgewählte Kommando
   * @param language
   *                 der Text wird in dieser Sprache angezeigt
   */
  public void setLanguage(KEY key, ISO639 language) {
    if (key == null) throw new IllegalArgumentException("key ist null");
    if (language == null) throw new IllegalArgumentException("language ist null");
    this.language = language;
    this.key = key;
    COMMAND command = Multilingual.getInstance().getCommand(this.key);
    LANGUAGES languages = command.getLANGUAGES();
    if (languages == null) return;
    for (LANGUAGE tmp : languages.getLANGUAGE()) {
      if (tmp.getISO639() != language) continue;
      if (tmp.getTEXT() != null) {
        renderer.setText(tmp.getTEXT());
      }
      break;
    }

  }





}
